#include "mylist.h"



/******************************************************************************
 * Bricht mit einer Fehlermeldung das Programm ab
 *****************************************************************************/
void err(char *s)
{
 printf(s);
 exit(1);
}


/******************************************************************************
 * Erstellt eine neue, leere Liste und liefert diese zurck.
 *****************************************************************************/
t_list* list_mkEmpty(void)
{
 return (t_list*) 0;
}



/******************************************************************************
 * Hngt ein neues Listenelement an die Liste list an und liefert die Liste
 * zurck.
 *****************************************************************************/
t_list* list_append(t_list* list, int num, char* text)
{

 /* neues Element erzeugen */
 t_list * res = malloc(sizeof(*list) );
 if (!res)   
   err("err01: Nicht gengend Speicher fr ein neues Listenelement!");

 /* Daten in das Element kopieren */
 res->data.num   = num;
 if (text)
   {
      res->data.text  = malloc(strlen(text)+1);
      if (!res->data.text)
         err("err02: Nicht gengend Speicher fr den Text im neuen Listenelement!");
      strcpy(res->data.text, text);
   }
 else
   res->data.text = text;

 /* Zeiger verbiegen */
 if (list)                          /* wenn mind. 1 Element in Liste */
   {
      res->next   = list->next;
      list->next  = res;
   }
 else                               /* Liste ist noch leer */
   {
      res->next   = res;
   }

 return res;
}




/******************************************************************************
 * Fgt ein Element hinter das Element after in die Liste list ein. 
 *****************************************************************************/
t_list* list_insert(t_list* list, t_list* after, int num, char* text)
{
 t_list * p;

 /* In einer leeren liste gibt es kein element,nichts tun */
 if (list==NULL && after!=NULL)
   return list;

 /* NULL ist bestimmt nicht in der liste enthalten, nichts tun */
 if (list!=NULL && after==NULL)
   return list;

 /* wenn das Element after in der liste list ist, dann einfuegen, sonst nix */
 if ( !list_inList(list, after) )            /* after nicht in Liste */
   return list;      
 
 p = list_append(after, num, text);          /* fge Element hinter after ein */
 return (list==after) ? p : list;            /* am Ende entspricht einem append */
}



/******************************************************************************
 * vergleicht Inhalt zweier Elemente
 *****************************************************************************/
int elemente_gleich(t_data e1, t_data e2)
{
 return  !strcmp(e1.text, e2.text)  &&  (e1.num == e2.num);
}



/******************************************************************************
 * Wenn das Element elem in der Liste list enthalten ist, liefert diese 
 * Funktion 1 zurck, sonst 0.
 *****************************************************************************/
int list_inList(t_list* list, t_list* elem)
{
 t_list * p = list;

 if (!list)                      /* leere Liste */
   return 0;

 do
   {
      p = p->next;

      if (p == elem)
         return 1;
   }
 while (p != list );             /* wieder beim letzten Element? */

 return 0;
}



/******************************************************************************
 * Prft, ob es ein Element mit dem Text text in list gibt. Gibt einen Zeiger 
 * auf das Element zurck, oder NULL. Existieren mehrere Elemente mit diesem 
 * Text, so wird das Element zurckgeliefert, das sich am weitesten vorne in 
 * der Liste befindet.
 *****************************************************************************/
t_list* list_findText(t_list* list, char* text)
{
 t_list * p = list;

 if (!list)                      /* leere Liste */
   return NULL;

 do
   {
      p = p->next;

      if (p->data.text && text && !strcmp(p->data.text, text)   )
         return p;
   }
 while (p != list );             /* wieder beim letzten Element? */

 return NULL;
}



/******************************************************************************
 * Tauscht die Reihenfolge der Elemente x und y in list und gibt den 
 * Listenzeiger zurck.
 *****************************************************************************/
t_list* list_exchange(t_list* list, t_list* x, t_list* y)
{
 t_data data; 
 
 /* wenn x und y beide in der Liste enthalten sind */
 if ( list_inList(list, x) && list_inList(list, y) )
   {
      data     = x->data;
      x->data  = y->data;
      y->data  = data;
   }

 return list;
}



/******************************************************************************
 * Hngt an den Text in dem Listenelement element den Text extratext an.
 *****************************************************************************/
void list_extendElement(t_list* element, char* extratext)
{
 char * p;

 if (!element || !extratext)          /* kein Element oder kein Text */
   return;

 p = malloc(strlen(element->data.text)+1 );

 strcpy(p, element->data.text);

 element->data.text = realloc(element->data.text, strlen(element->data.text)+strlen(extratext)+1 );
 if (!element->data.text)   
   err("err03: Nicht gengend Speicher fr ein neues Listenelement!");


 strcpy(element->data.text, p);
 strcat(element->data.text, extratext);
 free(p);
}



/******************************************************************************
 * Diese Funktion fgt die Listen list1 und list2 zusammen und liefert einen 
 * Listenzeiger zurck.
 *****************************************************************************/
t_list* list_concat(t_list* list1, t_list* list2)
{
 t_list * p;
 
 if (!list1)
   return list2;

 if (!list2)
   return list1;

 if (list1 == list2)
   return list1;

 p = list1->next;
 list1->next = list2->next;
 list2->next = p;
 return list2;
}



/******************************************************************************
 * Entfernt das Element element aus der Liste list und gibt dann den 
 * Listenzeiger zurck.
 *****************************************************************************/
t_list* list_remove(t_list* list, t_list* element)
{
 t_list * p = list, * res = list;


 if ( !list_inList(list, element) )
   return list;

 if (list == list->next)         /* nur das gesuchte El. in Liste */
   res = list_mkEmpty();
 else
   {
   do                            /* finde Listenel. vor element */
      {
         p = p->next;
      }
   while (p->next != element);

   if ( list == element)         /* element war war das letze Listenel. */
      res = p;  
   p->next = element->next;
   }
 
 free(element->data.text);
 free(element);

 return res;
}



/******************************************************************************
 * Lscht die gesamte Liste list. Benutzt die Funktion list_remove um das zu 
 * realisieren.
 *****************************************************************************/
void list_destroy(t_list* list)
{
 while (list)
   list = list_remove(list, list);
}

